@@ -2,10 +2,10 @@ require 'fog/core/collection'
 require 'fog/libvirt/models/compute/nic'
 
 module Fog
-  module Compute
-    class Libvirt
+  module Libvirt
+    class Compute
       class Nics < Fog::Collection
-        model Fog::Compute::Libvirt::Nic
+        model Fog::Libvirt::Compute::Nic
       end
     end
   end
