@@ -59,5 +59,25 @@ Shindo.tests('Fog::Compute[:libvirt] | server model', ['libvirt']) do
       end
     end
     test('be a kind of Fog::Libvirt::Compute::Server') { server.kind_of? Fog::Libvirt::Compute::Server }
+    tests("serializes to xml") do
+      test("with memory") { server.to_xml.match?(%r{<memory>\d+</memory>}) }
+      test("with disk of type file") do
+        xml = server.to_xml
+        xml.match?(/<disk type="file" device="disk">/) && xml.match?(%r{<source file="path/to/disk"/>})
+      end
+      test("with disk of type block") do
+        server = Fog::Libvirt::Compute::Server.new(
+          {
+            :nics => [],
+            :volumes => [
+              Fog::Libvirt::Compute::Volume.new({ :path => "/dev/sda", :pool_name => "dummy" })
+            ]
+          }
+        )
+        xml = server.to_xml
+        xml.match?(/<disk type="block" device="disk">/) && xml.match?(%r{<source dev="/dev/sda"/>})
+      end
+      test("with q35 machine type on x86_64") { server.to_xml.match?(%r{<type arch="x86_64" machine="q35">hvm</type>}) }
+    end
   end
 end
